﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Data;
using System.Data.SqlClient;
using YFDYFKnowledgeDiscovery.Dao;
using YFDYFKnowledgeDiscovery.Common;
using Excel = Microsoft.Office.Interop.Excel;
using System.Reflection;

namespace YFDYFKnowledgeDiscovery.Analysis
{
    public class SymptonDiseasePredictionAnalysis
    {
        static SqlConnection conn = DBHelper.getConnection();

        public static void test()
        {
            string sql = "select symptomdiseasepredictionid,kdsymptom from symptomdiseaseprediction";
            DataTable dt = DBHelper.ExecuteWithReset(sql, conn);
            for (int i = 0; i < dt.Rows.Count; i++)
            {
                string id = dt.Rows[i][0].ToString();
                string symptom = dt.Rows[i][1].ToString();
                List<string> diseaseList1 = Analysis.SymptonDiseasePredictionAnalysis.NaiveBayes(symptom);
                List<string> diseaseList2 = Analysis.SymptonDiseasePredictionAnalysis.statistic(symptom);
                string disease1Str = "";
                string disease2Str = "";
                if (diseaseList1 != null)
                {
                    for (int j = 0; j < diseaseList1.Count && j < 20; j++)
                    {
                        disease1Str += diseaseList1[j] + "；";
                    }
                }
                if (diseaseList2 != null)
                {
                    for (int j = 0; j < diseaseList2.Count && j < 20; j++)
                    {
                        disease2Str += diseaseList2[j] + "；";
                    }
                }
                sql = "update symptomdiseaseprediction set diseasePrediction1='" + disease1Str + "',diseasePrediction2='" + disease2Str + "' where symptomdiseasepredictionid='" + id + "'";
                int result = DBHelper.ExecuteNoResult(sql, conn);
            }
        }

        public static List<string> NaiveBayes(string userSymptom)
        {
            //根据用户输入症状字符串（每种症状用“、”隔开），形成用户输入症状列表
            String[] values = userSymptom.Split(new string[] { "、" }, StringSplitOptions.RemoveEmptyEntries);
            //获取所有能匹配的用户输入症状
            string sql = "select symptomname from kddiseasesymptom where ";
            for (int i = 0; i < values.Length; i++)
            {
                if (values[i].Equals("") == false)
                {
                    sql += " symptomname='" + values[i] + "' or ";
                }
            }
            sql += " 1=2 group by symptomname";
            DataTable dt = DBHelper.ExecuteWithReset(sql, conn);
            if (dt.Rows.Count == 0)
            {
                return null;
            }
            List<string> userSymptomList = new List<string>();
            for (int i = 0; i < dt.Rows.Count; i++)
            {
                userSymptomList.Add(dt.Rows[i][0].ToString());
            }
            //获取所有可能的疾病，确定范围
            sql = "select diseasename from kddiseasesymptom where ";
            for (int i = 0; i < userSymptomList.Count; i++)
            {
                sql += " symptomname='" + userSymptomList[i] + "' or ";
            }
            sql += " 1=2 group by diseasename";
            dt = DBHelper.ExecuteWithReset(sql, conn);
            List<string> diseaseList = new List<string>();
            for (int i = 0; i < dt.Rows.Count; i++)
            {
                diseaseList.Add(dt.Rows[i][0].ToString());
            }
            //根据可能疾病，获取所有可能症状
            List<string> symptomList = new List<string>();
            sql = "select symptomname from kddiseasesymptom where ";
            for (int i = 0; i < diseaseList.Count - 1; i++)
            {
                sql += " diseasename='" + diseaseList[i] + "' or ";
            }
            sql += " diseasename='" + diseaseList[diseaseList.Count - 1] + "' group by symptomname";
            dt = DBHelper.ExecuteWithReset(sql, conn);
            for (int i = 0; i < dt.Rows.Count; i++)
            {
                symptomList.Add(dt.Rows[i][0].ToString());
            }
            //得到可能疾病范围内的所有症状，以及这些症状与疾病的对应关系，形成映射矩阵
            sql = "select symptomname,diseasename from kddiseasesymptom where ";
            for (int i = 0; i < diseaseList.Count - 1; i++)
            {
                sql += " diseasename='" + diseaseList[i] + "' or ";
            }
            sql += " diseasename='" + diseaseList[diseaseList.Count - 1] + "'";
            dt = DBHelper.ExecuteWithReset(sql, conn);
            string[,] symptomDiseaseMap = new string[dt.Rows.Count, 2];
            for (int i = 0; i < dt.Rows.Count; i++)
            {
                symptomDiseaseMap[i, 0] = dt.Rows[i][0].ToString();
                symptomDiseaseMap[i, 1] = dt.Rows[i][1].ToString();
            }
            int symptomDiseaseMapLength = dt.Rows.Count;
            //在可能疾病的范围内，计算用户输入症状边缘概率
            double[] userSymptomProbability = new double[userSymptomList.Count];
            for (int i = 0; i < userSymptomList.Count; i++)
            {
                userSymptomProbability[i] = 0.0;
                for (int j = 0; j < symptomDiseaseMapLength; j++)
                {
                    if (symptomDiseaseMap[j, 0].Equals(userSymptomList[i]))
                    {
                        userSymptomProbability[i] += 1.0;
                    }
                }
                userSymptomProbability[i] = userSymptomProbability[i] / symptomDiseaseMapLength;
            }
            //在可能疾病的范围内，计算疾病边缘概率
            double[] diseaseProbability = new double[diseaseList.Count];
            for (int i = 0; i < diseaseList.Count; i++)
            {
                diseaseProbability[i] = 0.0;
                for (int j = 0; j < symptomDiseaseMapLength; j++)
                {
                    if (symptomDiseaseMap[j, 1].Equals(diseaseList[i]))
                    {
                        diseaseProbability[i] += 1.0;
                    }
                }
                diseaseProbability[i] = diseaseProbability[i] / symptomDiseaseMapLength;
            }
            //已知每种疾病的情况下，计算每种症状的条件概率
            double[,] userSymptomDiseaseProbability = new double[userSymptomList.Count, diseaseList.Count];
            for (int i = 0; i < diseaseList.Count; i++)
            {
                for (int j = 0; j < userSymptomList.Count; j++)
                {
                    userSymptomDiseaseProbability[j, i] = 0.0;
                }
            }
            for (int i = 0; i < diseaseList.Count; i++)
            {
                int tempNum = 0;
                for (int j = 0; j < symptomDiseaseMapLength; j++)
                {
                    if (symptomDiseaseMap[j, 1].Equals(diseaseList[i]) == true)
                    {
                        tempNum++;
                    }
                }
                for (int j = 0; j < symptomDiseaseMapLength; j++)
                {
                    if (symptomDiseaseMap[j, 1].Equals(diseaseList[i]) == true)
                    {
                        int index = userSymptomList.IndexOf(symptomDiseaseMap[j, 0]);
                        if(index >= 0)
                        {
                            userSymptomDiseaseProbability[index, i] = 1.0 / tempNum;
                        }
                    }
                }
            }
            //根据用户输入症状预测所有可能疾病概率,找出概率最大的疾病
            double[] diseaseUserSymptomProbability = new double[diseaseList.Count];
            for (int i = 0; i < diseaseList.Count; i++)
            {
                double temp1 = diseaseProbability[i]; //分子
                double temp2 = 1.0; //分母
                for (int j = 0; j < userSymptomList.Count; j++)
                {
                    temp1 = temp1 * userSymptomDiseaseProbability[j,i];
                    temp2 = temp2 * userSymptomProbability[j];
                }
                diseaseUserSymptomProbability[i] = temp1 / temp2;
            }
            //找出概率最大的疾病
            for (int i = 0; i < diseaseList.Count - 1; i++)
            {
                double maxProbability = diseaseUserSymptomProbability[i];
                int maxDiseaseIndex = i;
                for (int j = i + 1; j < diseaseList.Count; j++)
                {
                    if (maxProbability < diseaseUserSymptomProbability[j])
                    {
                        maxProbability = diseaseUserSymptomProbability[j];
                        maxDiseaseIndex = j;
                    }
                }
                if (maxDiseaseIndex != i)
                {
                    double tempDouble = diseaseUserSymptomProbability[i];
                    diseaseUserSymptomProbability[i] = diseaseUserSymptomProbability[maxDiseaseIndex];
                    diseaseUserSymptomProbability[maxDiseaseIndex] = tempDouble;
                    string tempStr = diseaseList[i];
                    diseaseList[i] = diseaseList[maxDiseaseIndex];
                    diseaseList[maxDiseaseIndex] = tempStr;
                }
            }
            if (diseaseUserSymptomProbability[0] != 0)
            {
                List<string> diseaseListNonZero = new List<string>();
                for (int i = 0; i < diseaseList.Count; i++)
                {
                    if (diseaseUserSymptomProbability[i] > 0)
                    {
                        diseaseListNonZero.Add(diseaseList[i]);
                    }
                }
                return diseaseListNonZero;
            }
            else
            {
                sql = "select diseasename,count(1) as a from kddiseasesymptom where ";
                for (int i = 0; i < values.Length; i++)
                {
                    sql += " symptomname='" + values[i] + "' or ";
                }
                sql += " 1=2 group by diseasename order by a desc";
                dt = DBHelper.ExecuteWithReset(sql, conn);
                if (dt.Rows.Count == 0)
                {
                    return null;
                }
                diseaseList = new List<string>();
                for (int i = 0; i < dt.Rows.Count; i++)
                {
                    diseaseList.Add(dt.Rows[i][0].ToString());
                }
                return diseaseList;
            }
        }

        public static List<string> statistic(string userSymptom)
        {
            //根据用户输入症状字符串（每种症状用“、”隔开），形成用户输入症状列表
            String[] values = userSymptom.Split(new string[] { "、" }, StringSplitOptions.RemoveEmptyEntries);
            //通过统计关联匹配个数，预测最有可能的疾病
            string sql = "select diseasename,count(1) as a from kddiseasesymptom where ";
            for (int i = 0; i < values.Length; i++)
            {
                if (values[i].Equals("") == false)
                {
                    sql += " symptomname='" + values[i] + "' or ";
                }
            }
            sql += " 1=2 group by diseasename order by a desc";
            DataTable dt = DBHelper.ExecuteWithReset(sql, conn);
            if (dt.Rows.Count == 0)
            {
                return null;
            }
            List<string> diseaseList = new List<string>();
            for (int i = 0; i < dt.Rows.Count; i++)
            {
                diseaseList.Add(dt.Rows[i][0].ToString());
            }
            return diseaseList;
        }
    }
}
