﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Configuration;
using System.Data.SqlClient;
using System.Data;
using System.Text.RegularExpressions;

namespace YFDYFKnowledgeDiscovery.Common
{
    /// <summary>
    /// 数据库连接字符串
    /// </summary>
    public class DBHelper
    {
        /// <summary>
        /// 获取数据库连接
        /// </summary>
        /// <returns></returns>
        public static SqlConnection getConnection()
        {
            try
            {
                string MSSqlDBConStr = "Data Source=49.123.83.186;Initial Catalog=YFDYF;User Id=sa;Password=12345678";
                if (string.IsNullOrEmpty(MSSqlDBConStr))
                {
                    return null;
                }
                else
                {
                    SqlConnection conn = new SqlConnection(MSSqlDBConStr);
                    if (conn.State == ConnectionState.Closed)
                        conn.Open();
                    return conn;
                }
            }
            catch (Exception ex)
            {
                return null;
            }

        }

        /// <summary>
        /// 返回影响记录的条数（如update、delete、alter等）
        /// 无结果集返回
        /// </summary>
        /// <param name="sql">待执行的SQL语句</param>
        /// <param name="conn">数据库连接</param>
        /// <returns>-1：执行错误 </returns>
        public static int ExecuteNoResult(string sql, SqlConnection conn)
        {
            if (conn == null)
            {
                return -1;
            }
            else
            {
                SqlCommand cmd = null;
                try
                {
                    if (conn.State == ConnectionState.Closed)
                        conn.Open();
                    cmd = new SqlCommand();
                    cmd.Connection = conn;
                    cmd.CommandText = sql;
                    cmd.CommandTimeout = 5000;//设置超时时间为5秒
                    int result = cmd.ExecuteNonQuery();
                    return result;

                }
                catch (Exception ex)
                {
                    return -1;
                }
                finally
                {
                    cmd.Dispose();
                }
            }
        }

        /// <summary>
        /// 执行SQL语句，并返回第一行第一列记录【如 select count(*)等】
        /// </summary>
        /// <param name="sql">待执行的SQL语句</param>
        /// <param name="conn">数据库连接</param>
        /// <returns>第一行第一列结果，null:执行错误</returns>
        public static object ExecuteWithOneRow(string sql, SqlConnection conn)
        {

            if (conn == null)
            {
                return null;
            }
            else
            {
                SqlCommand cmd = null;
                try
                {
                    if (conn.State == ConnectionState.Closed)
                        conn.Open();
                    cmd = new SqlCommand();
                    cmd.Connection = conn;
                    cmd.CommandText = sql;
                    cmd.CommandTimeout = 5000;//设置超时时间为5秒
                    object result = cmd.ExecuteScalar();
                    return result;

                }
                catch (Exception ex)
                {
                    return null;
                }
                finally
                {
                    cmd.Dispose();
                }
            }
        }

        /// <summary>
        /// 执行带返回结果的SQL语句
        /// </summary>
        /// <param name="sql">待执行的SQL语句</param>
        /// <param name="conn">数据库连接</param>
        /// <returns>null:执行错误</returns>
        public static DataTable ExecuteWithReset(string sql, SqlConnection conn)
        {
            if (conn == null)
            {
                return null;
            }
            else
            {
                SqlDataAdapter sda = null;
                try
                {
                    if (conn.State == ConnectionState.Closed)
                        conn.Open();
                    sda = new SqlDataAdapter(sql, conn);
                    DataTable result = new DataTable();
                    sda.Fill(result);
                    return result;

                }
                catch (Exception ex)
                {
                    return null;
                }
                finally
                {
                    sda.Dispose();
                }
            }
        }

        /// <summary>
        /// 执行SQL事务
        /// </summary>
        /// <param name="sqls">待执行的SQL语句集合</param>
        /// <param name="conn">数据库连接</param>
        /// <returns>true:执行成功 false:执行失败</returns>
        public static bool ExecuteTransaction(List<string> sqls, SqlConnection conn)
        {
            if (conn == null)
                return false;
            else
            {
                SqlCommand cmd = null;
                SqlTransaction tran = null;
                try
                {
                    if (conn.State == ConnectionState.Closed)
                        conn.Open();
                    cmd = new SqlCommand();
                    tran = conn.BeginTransaction();
                    cmd.Connection = conn;
                    cmd.Transaction = tran;
                    foreach (string sql in sqls)
                    {
                        cmd.CommandText = sql;
                        cmd.ExecuteNonQuery();
                    }
                    tran.Commit();
                    return true;
                }
                catch (Exception ex)
                {
                    tran.Rollback();//事务回滚
                    return false;
                }
                finally
                {
                    tran.Dispose();
                    cmd.Dispose();
                }

            }
        }

        /// <summary>
        /// 关闭数据库连接
        /// </summary>
        /// <param name="conn"></param>
        public static void CloseConnection(SqlConnection conn)
        {
            try
            {
                if (conn != null && conn.State == ConnectionState.Open)
                    conn.Close();
            }
            catch (Exception ex)
            {
            }
            finally
            {
                conn.Dispose();
            }
        }

        /// <summary>
        /// 执行存储过程（无返回值）
        /// </summary>
        /// <param name="executeStoredName"></param>
        /// <param name="conn"></param>
        /// <returns></returns>
        public static void ExcuteStoredProcedureNoResult(string StoredName, SqlConnection conn)
        {
            if (conn == null)
            {
                return ;
            }
            else
            {
                SqlCommand cmd = null;
                try
                {
                    if (conn.State == ConnectionState.Closed)
                        conn.Open();
                    cmd = new SqlCommand();
                    cmd.Connection = conn;
                    cmd.CommandType = CommandType.StoredProcedure;
                    cmd.CommandText = StoredName;
                    cmd.CommandTimeout = 5000;//设置超时时间为5秒
                    cmd.ExecuteNonQuery();
                }
                catch (Exception ex)
                {
                    return ;
                }
                finally
                {
                    cmd.Dispose();
                }
            }
        
        }
        /// <summary>
        /// 执行带返回参数的存储过程
        /// </summary>
        /// <param name="StoredName"></param>
        /// <param name="conn"></param>
        /// <param name="resultName"></param>
        /// <param name="type"></param>
        /// <returns></returns>
        public static int ExcuteStoredProcedureWithResult(string StoredName,
            SqlConnection conn,
            List<SqlParameter> InparamList,ref List<SqlParameter> outParamList)
        {
            if (conn == null)
            {
               
                return -1;
            }
            else
            {
                SqlCommand cmd = null;
                try
                {
                    if (conn.State == ConnectionState.Closed)
                        conn.Open();
                    cmd = new SqlCommand();
                    cmd.Connection = conn;
                    cmd.CommandType = CommandType.StoredProcedure;
                    cmd.CommandText = StoredName;
                    foreach (SqlParameter tempparam in InparamList)
                    {
                        tempparam.Direction = ParameterDirection.Input;
                        cmd.Parameters.Add(tempparam);
                    }
                    if (outParamList != null)
                    {
                        foreach (SqlParameter tempparam in outParamList)
                        {
                            tempparam.Direction = ParameterDirection.Output;
                            cmd.Parameters.Add(tempparam);
                        }
                    }
                    SqlParameter returnParam = new SqlParameter("@return", SqlDbType.Int);
                    returnParam.Direction = ParameterDirection.ReturnValue;
                    cmd.Parameters.Add(returnParam);
                    cmd.CommandTimeout = 5000;//设置超时时间为5秒
                    cmd.ExecuteNonQuery();
                    return Convert.ToInt32(returnParam.Value.ToString());
                }
                catch (Exception ex)
                {
                    return -1;
                }
                finally
                {
                    cmd.Dispose();
                }
            }

        }




        public static string EncodeInputString(string inputString)
        {
            //要替换的敏感字
            inputString=inputString.Replace("-","");
            string SqlStr = @"and|or|exec|execute|insert|select|delete|update|alter|create|drop|count|\*|chr|char|asc|mid|substring|master|truncate|declare|xp_cmdshell|restore|backup|net +user|net +localgroup +administrators";
            try
            {
                if ((inputString != null) && (inputString != String.Empty))
                {
                    string str_Regex = @"\b(" + SqlStr + @")\b";
                    Regex Regex = new Regex(str_Regex, RegexOptions.IgnoreCase);
                    //string s = Regex.Match(inputString).Value; 
                    MatchCollection matches = Regex.Matches(inputString);
                    for (int i = 0; i < matches.Count; i++)
                        inputString = inputString.Replace(matches[i].Value, "[" + matches[i].Value + "]");

                }
            }
            catch
            {
                return "";
            }
            return inputString;
        }
    }
}