﻿using System;
using System.Collections.Generic;
using System.Data.SqlClient;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using YFDYFKnowledgeDiscovery.Common;

namespace YFDYFKnowledgeDiscovery.Semantics
{
    /// <summary>
    /// 评价整合
    /// </summary>
    public class CommentIntegration
    {
        //获取数据库连接
        static SqlConnection conn = DBHelper.getConnection();
        /// <summary>
        /// 整合京东数据
        /// </summary>
        public void IntegrationJD()
        {
            try
            {
                //获取数据
                string getDataSql = "SELECT distinct(comment),name FROM JD where comment is not null";
                var result = DBHelper.ExecuteWithReset(getDataSql, conn);
                //更新数据
                //IntegrationModel model = new IntegrationModel();
                for (int i = 0; i < result.Rows.Count; i++)
                {
                    string[] comments = result.Rows[i]["comment"].ToString().Split('@');
                    string goodName = result.Rows[i]["name"].ToString();
                    List<string> sqls = new List<string>();
                    foreach (string comment in comments)
                    {
                        if (!string.IsNullOrEmpty(comment.Trim()))
                        {
                            string[] sArr = comment.Replace("&hellip;", "").Split(new string[] { "||" }, StringSplitOptions.RemoveEmptyEntries);
                            IntegrationModel model = new IntegrationModel();
                            model.GoodName=goodName;
                            if (sArr.Length >= 2)
                            {
                                model.Comments = sArr[0].Trim('|');
                                model.Scores = Convert.ToDouble(sArr[1].ToString().Trim('|'));
                            }
                            sqls.Add(InsertIntegratedDataSql(model));
                        }
                    }
                    DBHelper.ExecuteTransaction(sqls, conn);
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message); // throw  
            }   
        }
        /// <summary>
        /// 整合天猫数据
        /// </summary>
        public void IntegrationTMall()
        {
            try
            {
                //获取数据
                string getDataSql = "SELECT distinct(comment),name FROM TMall where id<850000 and id>=700000 and comment is not null and comment!=''";
                var result = DBHelper.ExecuteWithReset(getDataSql, conn);
                //更新数据
                //IntegrationModel model = new IntegrationModel();
                for (int i = 0; i < result.Rows.Count; i++)
                {
                    string[] comments = result.Rows[i]["comment"].ToString().Split('@');
                    string goodName = result.Rows[i]["name"].ToString();
                    List<string> sqls = new List<string>();
                    foreach (string comment in comments)
                    {
                        if (!string.IsNullOrEmpty(comment.Trim()))
                        {
                            IntegrationModel model = new IntegrationModel()
                            {
                                GoodName = goodName,
                                Comments = comment,
                                Scores = -1.0f
                            };
                            sqls.Add(InsertIntegratedDataSql(model));
                        }
                    }
                    DBHelper.ExecuteTransaction(sqls, conn);
                }
            }
            catch (Exception ex)
            {

            }
        }
        /// <summary>
        /// 整合一号店数据
        /// </summary>
        public void IntegrationYhd()
        {
            try
            {
                //获取数据
                string getDataSql = "SELECT comment,name FROM YiHaoDian where LEN(comment)>0";
                var result = DBHelper.ExecuteWithReset(getDataSql, conn);
                //更新数据
                //IntegrationModel model = new IntegrationModel();
                for (int i = 0; i < result.Rows.Count; i++)
                {
                    string[] comments = result.Rows[i]["comment"].ToString().Split('@');
                    string goodName = result.Rows[i]["name"].ToString();
                    List<string> sqls = new List<string>();
                    foreach (string comment in comments)
                    {
                        if (!string.IsNullOrEmpty(comment.Trim()))
                        {
                            IntegrationModel model = new IntegrationModel();
                            string[] sArr = comment.Split(new string[] { "||" }, StringSplitOptions.RemoveEmptyEntries);
                            model.GoodName = goodName;
                            foreach (var c in sArr)
                            {
                                if (c.Contains("【用户名】"))
                                {
                                }
                                else if (c.Contains("【评分】"))
                                {
                                    model.Scores = getScoreByStar(c.Trim(new char[] { '【', '评', '分', '】' }));
                                }
                                else //评价
                                {
                                    model.Comments = c.Trim(new char[] { '【', '评', '论', '】' });
                                }
                            }

                            //sqls.Add(InsertIntegratedDataSql(model));
                            DBHelper.ExecuteNoResult(InsertIntegratedDataSql(model), conn);
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                throw new Exception(ex.Message); // throw  
            }     
        }
        /// <summary>
        /// 拼接插入商品评价sql语句
        /// </summary>
        /// <param name="model"></param>
        string InsertIntegratedDataSql(IntegrationModel model)
        {
            string sql = "insert into goodComments(goodName,comments,scores,userName,remark) values('" + model.GoodName + "','" + model.Comments + "','" + model.Scores + "','" + model.UserName + "','" + model.Remark + "')";
            return sql;
        }

        double getScoreByStar(string star)
        {
            switch (star)
            {
                case "s5":
                    return 5;
                case "s4":
                    return 4;
                case "s3":
                    return 3;
                case "s2":
                    return 2;
                case "s1":
                    return 1;
                default:
                    return -3;
            }
        }

        class IntegrationModel
        {
            /// <summary>
            /// 商品名称
            /// </summary>
            public string GoodName { set; get; }
            /// <summary>
            /// 商品评论
            /// </summary>
            public string Comments { set; get; }
            /// <summary>
            /// 商品评分
            /// </summary>
            public double Scores { set; get; }
            /// <summary>
            /// 评论者姓名
            /// </summary>
            public string UserName { set; get; }
            /// <summary>
            /// 备注
            /// </summary>
            public string Remark { set; get; }
        }
    }
}
